/*************************************************************/
/*  mandtest.c Mandelbrot program                            */
/*                                                           */
/*  Latest update: June 98, Stefan Spaennare                 */
/*************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

double timetic=(double)(CLOCKS_PER_SEC);

int iter(c1,c2,itmax)
double c1,c2;
int itmax;
{
   double z1,z2,z3;
   int it;

   z1=0.0;
   z2=0.0;
   it=0;

   while ((it < itmax) && ((z1*z1+z2*z2) <= 4.0)) {
      z3=z1;
      z1=z1*z1-z2*z2+c1;
      z2=2.0*z3*z2+c2;
      it++;
   } 
   return(it);
} /* end iter */


int main(argc,argv)
int argc;
char *argv[];
{
  
   FILE *outfile;

   int i,j,itmax,it,xsize,ysize;

   double a,b,da,db,ca,cb,wa,wb,flops,ftime;

   int xys[3];

   short fff[3];

   short **image;
   short *mpimage;

   clock_t time1,timediff;

   if (argc<2) {
      printf("Usage: %s image.sss <parameterlist\n",argv[0]);
      printf("Parameters: ar war br wbr itmaxi xsizei ysizei\n");
      exit(0);
   } /* if */
  
   scanf("%lf %lf %lf %lf %d %d %d",&a,&wa,&b,&wb,&itmax,&xsize,&ysize); 

   image=(short **)calloc(ysize,sizeof(short*));
   mpimage=(short *)calloc(xsize*ysize,sizeof(short));

   for (i=0; i<ysize; i++) {
      image[i]=mpimage;
      mpimage=mpimage+xsize;
   } /* for i */

   da=wa/(double)(xsize);
   db=wb/(double)(ysize);


   flops=0.0;

   time1=clock();

   for (i=0; i<ysize; i++) {
      for (j=0; j<xsize; j++) {
         ca=a+da*(double)(j);
         cb=b+db*(double)(i);
         it=iter(ca,cb,itmax); 
	 image[i][j]=it;
         flops=flops+10.0*(double)(it)+7.0;
      } /* for j */
   } /* for i */


   timediff=clock()-time1;

   ftime=(double)(timediff)/timetic;
   flops=flops/ftime;

   printf("\nTime:  %12.2f\n",ftime);
   printf("Flops: %12.2f\n\n",flops);

   xys[1]=xsize;
   xys[2]=ysize;

   outfile=fopen(argv[1],"wb");

   fwrite(&xys[1],sizeof(int),2,outfile);

   for (i=0; i<ysize; i++) {
      for (j=0; j<xsize; j++) {
         fff[1]=image[i][j];
         fwrite(&fff[1],sizeof(short),1,outfile);
      } /* for j */
   } /* for i */

   fclose(outfile);

} /* End */
