/******************************************************/
/*                                                    */
/* This program is free to copy !                     */
/*                                                    */
/* Program: primm2.c                                  */
/*                                                    */
/* This program calculates the number of prime        */
/* numbers below n.                                   */
/*                                                    */
/* Author: Stefan Spaennare                           */
/* Email:  stefans@astro.lu.se                        */
/*                                                    */
/* Memory requirement: 4*(1.25*n/ln(n)+1000) bytes    */
/*                                                    */
/* Benchmarking:                                      */
/*                                                    */
/* n = 1000000 = 10^6                                 */
/*                                                    */
/* Computer:       CPU-time (s):      Compilation:    */
/*                                                    */
/* HP 735 99 MHz       13.82            cc  +O3       */
/* Pentium 150 MHz      5.99            gcc -O4       */
/*                                                    */
/******************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

double timetic=(double)(CLOCKS_PER_SEC);

int main(argc,argv)
int argc;
char *argv[];
{
   int *p;

   int i,j,k,i2,n,p2,m,l,sum;

   clock_t time1,timediff;

   if (argc < 2) {
      printf("Usage: %s ni\n",argv[0]);
      exit(0);
   } /* if */

   n=atoi(argv[1]);

   l=(int)(1.25*(double)(n)/log((double)(n))+0.5)+1000;

   p=(int *)calloc(l,sizeof(int));

   p[1]=2;
   p[2]=3;
   p[3]=5;

/*   printf("       2       3       5"); */

   sum=3;

   j=3;
   m=3;

   time1=clock();

   for (i=7; i<=n; i++) {
      i2=(int)(sqrt((double)(i)))+1;
      k=1;
      p2=p[k];
      while ((p2<i2) && ((i % p2) != 0)) {
         k++;
         p2=p[k];
      } /* while */
      if (p2>=i2) {
/*
         printf("%8d",i); 
         m++;
         if (((m) % 8) == 0) {
           printf("\n");
         } 
*/
         sum++;
         j++;
         p[j]=i;
      } /* if */
   } /* for i */

   timediff=clock()-time1;

   printf("%12d\n\n",sum);

   printf("%12.2f\n",(double)(timediff)/timetic);

} /* End */
